@@ -30,6 +30,7 @@
 #include "modulemanager.h"
 #include "profile.h"
 #include "port.h"
+#include "volumemonitor.h"
 
 #include "globalactioncollection.h"
 #include "microphoneindicator.h"
@@ -57,6 +58,7 @@ void Plugin::registerTypes(const char* uri)
     qmlRegisterType<QPulseAudio::SourceOutputModel>(uri, 0, 1, "SourceOutputModel");
     qmlRegisterType<QPulseAudio::StreamRestoreModel>(uri, 0, 1, "StreamRestoreModel");
     qmlRegisterType<QPulseAudio::ModuleModel>(uri, 0, 1, "ModuleModel");
+    qmlRegisterType<QPulseAudio::VolumeMonitor>(uri, 0, 01, "VolumeMonitor");
     qmlRegisterUncreatableType<QPulseAudio::Profile>(uri, 0, 1, "Profile", QString());
     qmlRegisterUncreatableType<QPulseAudio::Port>(uri, 0, 1, "Port", QString());
     qmlRegisterType<GlobalAction>(uri, 0, 1, "GlobalAction");
@@ -70,13 +72,8 @@ void Plugin::registerTypes(const char* uri)
             Q_UNUSED(jsEngine);
             return new MicrophoneIndicator();
     });
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-    qmlRegisterType<QPulseAudio::Client>();
-    qmlRegisterType<QPulseAudio::Sink>();
-    qmlRegisterType<QPulseAudio::Source>();
-#else
     qmlRegisterAnonymousType<QPulseAudio::Client>(uri, 1);
     qmlRegisterAnonymousType<QPulseAudio::Sink>(uri, 1);
     qmlRegisterAnonymousType<QPulseAudio::Source>(uri, 1);
-#endif
+    qmlRegisterAnonymousType<QPulseAudio::VolumeObject>(uri, 1);
 }
